/* 18/12/2009: LN@IASF-INAF

   Note: input and oputput coordinates are in "degrees"

  LN, last changed: 09/06/2010

*/

#include <math.h>

#include "watdefs.h"
#include "afuncs.h"

//#define PI 3.1415926535897932384626433
#define PI2 6.2831853071795864769252867665590057683943387987502
#define RAD2DEG 57.295779513082320876798155
#define DEG2RAD 1.74532925199432957692369E-2

static const double MEQ2GL[9] = {  /* Equatorial 2000.0 -> Galactic */
      -.054875539726, -.873437108010, -.483834985808,
       .494109453312, -.444829589425,  .746982251810,
      -.867666135858, -.198076386122,  .455983795705 };

/*--> Unused
int DLL_FUNC precess_ra_dec2Gl( const double DLLPTR *matrix,
                        double *p_out,
                        const double DLLPTR *p_in, int backward)
{
   double v1[3], v2[3];
   const double old_ra = p_in[0];

   v1[0] = cos( p_in[0]) * cos( p_in[1]);
   v1[1] = sin( p_in[0]) * cos( p_in[1]);
   v1[2] =                 sin( p_in[1]);
   if( backward)
      deprecess_vector( matrix, v1, v2);
   else
      precess_vector( matrix, v1, v2);

   if( v2[1] || v2[0])
      *p_out = atan2( v2[1], v2[0]);
   else
      *p_out = 0.;

   while( *p_out - old_ra > PI)
      *p_out -= PI * 2.;
   while( *p_out - old_ra <-PI)
      *p_out += PI * 2.;

   *p_out *= RAD2DEG;
   return( 0);
}


int DLL_FUNC precess_ra_dec2Gb( const double DLLPTR *matrix,
                        double *p_out,
                        const double DLLPTR *p_in, int backward)
{
   double v1[3], v2[3];

   v1[0] = cos( p_in[0]) * cos( p_in[1]);
   v1[1] = sin( p_in[0]) * cos( p_in[1]);
   v1[2] =                 sin( p_in[1]);
   if( backward)
      deprecess_vector( matrix, v1, v2);
   else
      precess_vector( matrix, v1, v2);

   *p_out = asine( v2[2]);

   *p_out *= RAD2DEG;
   return( 0);
}
<--*/

double DLL_FUNC ra_dec2Gl( const double ra, const double dec)
{
   double opt;
   double v1[3], v2[3];
   const double rarad = ra*DEG2RAD, decrad=dec*DEG2RAD;
//   const double DLLPTR *matrix = j2000_to_galactic_matrix( );
   const double DLLPTR *matrix = MEQ2GL;
   //const double old_ra = rarad;

   v1[0] = cos( rarad) * cos( decrad);
   v1[1] = sin( rarad) * cos( decrad);
   v1[2] =               sin( decrad);

   precess_vector( matrix, v1, v2);

   if( v2[1] || v2[0])
      opt = atan2( v2[1], v2[0]);
   else
      return (0.);
      //opt = 0.;
/*
   while( (opt - old_ra) > PI)
      opt -= PI2;
   while( (opt - old_ra) <-PI)
      opt += PI2;

   if (opt > PI2)
     opt -= PI2;
*/
   opt = fmod(opt, PI2);
   if (opt < 0.)
     opt += PI2;

   return (opt * RAD2DEG);
}

double DLL_FUNC ra_dec2Gb( const double ra, const double dec)
{
   double v1[3], v2[3];
   const double rarad = ra*DEG2RAD, decrad=dec*DEG2RAD;
//   const double DLLPTR *matrix = j2000_to_galactic_matrix( );
   const double DLLPTR *matrix = MEQ2GL;

   v1[0] = cos( rarad) * cos( decrad);
   v1[1] = sin( rarad) * cos( decrad);
   v1[2] =               sin( decrad);

   precess_vector( matrix, v1, v2);

   return (asine( v2[2]) * RAD2DEG);
}
