/*
  LN @IASF-INAF, Dec 2009                     Last changed: 04/06/2010
  

  CREATE FUNCTION skysepc RETURNS REAL SONAME 'udf_astro.so';
  CREATE FUNCTION radec2gl RETURNS REAL SONAME 'udf_astro.so';
  CREATE FUNCTION radec2gb RETURNS REAL SONAME 'udf_astro.so';
  CREATE FUNCTION radec2el RETURNS REAL SONAME 'udf_astro.so';
  CREATE FUNCTION radec2eb RETURNS REAL SONAME 'udf_astro.so';
*/

#ifdef STANDARD
#include <stdio.h>
#include <string.h>
#ifdef __WIN__
typedef unsigned __int64 ulonglong;
typedef __int64 longlong;
#else
typedef unsigned long long ulonglong;
typedef long long longlong;
#endif /*__WIN__*/
#else
#include <my_global.h>
#include <my_sys.h>
#endif
#include <mysql.h>
#include <m_ctype.h>
#include <m_string.h>

#include "udf_astro.h"

#ifdef HAVE_DLOPEN



//extern "C" {
// External functions
extern double skysep_h(double theta1, double phi1,
                double theta2, double phi2,
                int coord_type);
extern double ra_dec2Gl( const double ra, const double dec);
extern double ra_dec2Gb( const double ra, const double dec);
extern double ra_dec2El( const double ra, const double dec);
extern double ra_dec2Eb( const double ra, const double dec);

// Local functions
DEFINE_FUNCTION(double, skysepc);
DEFINE_FUNCTION(double, radec2gl);
DEFINE_FUNCTION(double, radec2gb);
DEFINE_FUNCTION(double, radec2el);
DEFINE_FUNCTION(double, radec2eb);

//}

/* Angular distance on the sphere (in "arcmin") */

my_bool skysepc_init(UDF_INIT* init, UDF_ARGS *args, char *message)
{
  const char* argerr = "skysepc(Ra1_deg DOUBLE, Dec1_deg DOUBLE, Ra2_deg DOUBLE, Dec2_deg DOUBLE)";

  CHECK_ARG_NUM(4);
  CHECK_ARG_NOT_TYPE(0, STRING_RESULT);
  CHECK_ARG_NOT_TYPE(1, STRING_RESULT);
  CHECK_ARG_NOT_TYPE(2, STRING_RESULT);
  CHECK_ARG_NOT_TYPE(3, STRING_RESULT);

  return 0;
}


double skysepc(UDF_INIT *init, UDF_ARGS *args,
                char *is_null, char* error)
{
  return skysep_h(DARGS(0), DARGS(1), DARGS(2), DARGS(3), 0);
}


void skysepc_deinit(UDF_INIT *init)
{}


/* Equatorial 2000 to Galactic 2000 */

my_bool radec2gl_init(UDF_INIT* init, UDF_ARGS *args, char *message)
{
  const char* argerr = "radec2gl(Ra_deg DOUBLE, Dec_deg DOUBLE)";

  CHECK_ARG_NUM(2);
  CHECK_ARG_NOT_TYPE(0, STRING_RESULT);
  CHECK_ARG_NOT_TYPE(1, STRING_RESULT);

  init->ptr = NULL;

  return 0;
}


double radec2gl(UDF_INIT *init, UDF_ARGS *args,
                char *is_null, char* error)
{
  return ra_dec2Gl(DARGS(0), DARGS(1));
}


void radec2gl_deinit(UDF_INIT *init)
{}


my_bool radec2gb_init(UDF_INIT* init, UDF_ARGS *args, char *message)
{
  const char* argerr = "radec2gb(Ra_deg DOUBLE, Dec_deg DOUBLE)";

  CHECK_ARG_NUM(2);
  CHECK_ARG_NOT_TYPE(0, STRING_RESULT);
  CHECK_ARG_NOT_TYPE(1, STRING_RESULT);

  init->ptr = NULL;

  return 0;
}


double radec2gb(UDF_INIT *init, UDF_ARGS *args,
                char *is_null, char* error)
{
  return ra_dec2Gb(DARGS(0), DARGS(1));
}


void radec2gb_deinit(UDF_INIT *init)
{}

/* Equatorial 2000 to Ecliptic 2000 */

my_bool radec2el_init(UDF_INIT* init, UDF_ARGS *args, char *message)
{
  const char* argerr = "radec2el(Ra_deg DOUBLE, Dec_deg DOUBLE)";

  CHECK_ARG_NUM(2);
  CHECK_ARG_NOT_TYPE(0, STRING_RESULT);
  CHECK_ARG_NOT_TYPE(1, STRING_RESULT);

  init->ptr = NULL;

  return 0;
}


double radec2el(UDF_INIT *init, UDF_ARGS *args,
                char *is_null, char* error)
{
  return ra_dec2El(DARGS(0), DARGS(1));
}


void radec2el_deinit(UDF_INIT *init)
{}


my_bool radec2eb_init(UDF_INIT* init, UDF_ARGS *args, char *message)
{
  const char* argerr = "radec2eb(Ra_deg DOUBLE, Dec_deg DOUBLE)";

  CHECK_ARG_NUM(2);
  CHECK_ARG_NOT_TYPE(0, STRING_RESULT);
  CHECK_ARG_NOT_TYPE(1, STRING_RESULT);

  init->ptr = NULL;

  return 0;
}


double radec2eb(UDF_INIT *init, UDF_ARGS *args,
                char *is_null, char* error)
{
  return ra_dec2Eb(DARGS(0), DARGS(1));
}


void radec2eb_deinit(UDF_INIT *init)
{}


#endif /* HAVE_DLOPEN */
