/*
   getcat.c

  Purpose:

   Read MySQL catalogues from a specific DB. See "MyServer.h".

   This version requires a full SQL string to be passed to MySQL.
   Output is what you would have if issuing that query within a mysql client.

   LN@INAF-IASF, December 2014                   ( Last change: 03/12/2014 )
*/

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>
#include "my_db.h"
#include "MyServer.h"  // edit this for your case

// Version ID string
static char* VERID="Ver 0.1a, 03-12-2014, LN@INAF-IASF";

int
main(int argc,char *argv[]) {
  unsigned int i, j;
  int ret;
  char *p;

  if (argc != 2) { 
    printf("%s  %s\n\n",argv[0], VERID);

    printf("Usage:   %s \"SQL_query\"\n\n", argv[0]);
    printf("Example: %s \"show tables\"\n", argv[0]);
    printf("         %s \"describe UCAC_2orig\"\n", argv[0]);
    printf("         %s \"SELECT COUNT(*) from UCAC_2orig\"\n\n", argv[0]);
    printf("         %s \"select * from UCAC_2orig where (htmID_6=32768 and RAcs< 9000)\"\n", argv[0]);
    printf("         %s \"select RAcs,DECcs,Amm from UCAC_2orig where (htmID_6=32768 and DECcs > -3000)\"\n", argv[0]);
    printf("         %s \"select RAcs/3.6e5 as RA, DECcs/3.6e5 as DE, Amm/1000 as A_mag form UCAC_2orig where (htmID_6=32768 and DECcs > -3000)\"\n\n", argv[0]);
    printf("Note: You must enclose the query in double quotes and use single\n");
    printf("      quotes for strings in SQL queries.\n");
    return(1);
  }

  ret = db_init(0);
  if (!ret) { printf("Can't set CONNECT_TIMEOUT for MySQL connection.");  return(1); }

//db_connect(0, hostname, username, password, dbname), returns 0 on error
//  ret=db_connect(0, "localhost", "generic", "password", "RossCats");
//  ret=db_connect(0, "192.167.166.42", "generic", "password", "RossCats");
  ret = db_connect(0, SERVER_IP,SERVER_USER,SERVER_PWD,SERVER_DB);

  if (!ret) { printf("DB error: %s\n", db_error(0));  return(1); }

  ret = db_query(0, argv[1]);
  if (!ret) { printf("DB error: %s\n", db_error(0));  return(1); }

  if (db_return_row(0)) { 
    for(j = 0; j < db_num_fields(0); j++)
      printf("%s\t", db_fieldname(0, j));
    
    printf("\n");
    
    for(j = 0; j < db_num_fields(0); j++) {
      p = db_fieldname(0, j);
      for(i = 0; i < strlen(p); i++)
	printf("-");
      
      printf("\t");
    }
    printf("\n");
    
    for (j = 0; j < db_num_rows(0); j++) {
      for(i = 0; i < db_num_fields(0); i++)
	printf("%s\t", db_data(0, j, i));
      
      printf("\n");
    }
  }
  else 
    printf("\n%d rows affected\n", db_num_rows(0));

  db_close(0);
  return(0);
}
