/*
   getcat_radec2_BSC.c

  Purpose:

   Read custom MySQL DB Bright Star Catalogue. See "MyServer.h".

   This version only requires the RA, DEC corners (in deg.) of a rectangle,
   or the center of a square and its side in arcmin.

  Usage (corners and center coord. in degrees):
        getcat_radec2_BSC  RAcorner1 DECcorner1 RAcorner2 DECcorner2
     or:
        getcat_radec2_BSC RAcenter DECcenter side_arcmin
     or:
        getcat_radec2_BSC hh mm ss (RAcenter) dd mm ss (DECcenter) side_arcmin

  Example:
      ./getcat_radec2_BSC 20 -10 500

   Returns selected data for the objects found in that rectangle/square.
   Output is NOT sorted.

   LN@IASF-CNR, June 2003                   ( Last change: 07/12/2014 )

*/

#include <stdlib.h> 
#include <stdio.h> 
#include <unistd.h>
#include <string.h>
#include "my_db.h"
#include "MyServer.h"

extern char *getcat_buildq_BSC(int ac, char *av[]);
extern int praws_BSC();

// Number of required arguments (could be 3)
#define NVALS   4

void
usage(char *name) {

  printf("Usage (corners and center coord. in degrees):\n");
  printf("  %s RAcorner1 DECcorner1 RAcorner2 DECcorner2\n",name);
  printf("or:\n");
  printf("  %s RAcenter DECcenter side_arcmin\n",name);
  printf("or:\n");
  printf("  %s hh mm ss (RAcenter) dd mm ss (DECcenter) side_arcmin\n",name);
  exit(0);
}

int
main(int argc,char *argv[]) {
  unsigned int ac;
  int ret;

  char **av, *comm;

/* There must be at least 3 args. */
  ac = --argc;
  if (ac != NVALS)
  {
    if (ac != 3 && ac != 7)
    {
      usage(argv[0]);
      return(0);
    }
  }

  av = ++argv;
  comm = getcat_buildq_BSC(ac, av);

  if (strcmp(comm,"\0") == 0)
  {
    printf("getcat_buildq_BSC: Error encoding query!\n");
    exit(1);
  }

printf("Query:\n%s\n\n",comm);

  ret=db_init(0);
  if (!ret) { printf("Can't set CONNECT_TIMEOUT for MySQL connection.");
    return(1); }

//db_connect(0, hostname, username, password, dbname), returns 0 on error
//  ret=db_connect(0, "localhost", "generic", "password", "MyCats");
  ret=db_connect(0, SERVER_IP,SERVER_USER,SERVER_PWD,SERVER_DB);

  if (!ret) { printf("DB error: %s\n", db_error(0));  return(1); }

  ret=db_query(0, comm);
  if (!ret) { printf("DB error: %s\n", db_error(0));  return(1); }

  if (db_return_row(0))
  {

/* Print header - it's done in praws_BSC() */
/*
    printf(" HR      Name     IRflag  Multiple  ADScomp    VarID       RA          DEC      GLON    GLAT    Vmag   BmV    UmB    RmI          SpType          RaPM   DecPM   Parallax  MultCnt\n"
           "----  ----------  ------  --------  -------  ---------  ----------  ---------  ------  ------  -----  -----  -----  -----  --------------------  ------  ------  --------  -------\n");

    for(i = 0; i < db_num_fields(0); i++)
      printf("%s\t", db_fieldname(0, i));
    printf("\n");

    for(j = 0; j < db_num_fields(0); j++) {
      p=db_fieldname(0, j);
      for(i = 0; i < strlen(p); i++)
	printf("-");
      printf("\t");
    }
    printf("\n");
*/

/* Print the data */
    ret=praws_BSC();
    if (ret != 0) printf("\npraws_BSC reported an error.");
  }
  else 
    printf("\n%d rows affected\n", db_num_rows(0));

  db_close(0);
  return(0);
}
