PRO AITOFF_GRID2,DLONG,DLAT,LINESTYLE=N, LINECOLOR=linecolor, CHRTHK=chrthk, $
                 CHRCOL=chrcol, CHRSIZ=chrsiz, LABEL=label, _extra=e
;+
; NAME:
;	AITOFF_GRID2
;
; PURPOSE:
;	Produce an overlay of latitude and longitude lines over a plot or image
;	on the current graphics device. AITOFF_GRID2 assumes that the ouput plot
;	coordinates span the x-range of -180 to 180 and the y-range goes from
;	-90 to 90.
;
; CALLING SEQUENCE:
;
;	AITOFF_GRID2 [,DLONG,DLAT]
;
; INPUTS:
;
;	DLONG	= Optional input longitude line spacing in degrees. If left
;		  out, defaults to 30.
;       DLAT    = Optional input lattitude line spacing in degrees. If left
;                 out, defaults to 30.
;
; KEYWORDS:
;
;	LINESTYLE	= Input integer specifying the linestyle to
;			  use for drawing the grid lines.
;	LINECOLOR	= Input integer specifying the color to
;			  use for drawing the grid lines.
;	LABELS		= Keyword specifying that the lattitude and
;			  longitude lines on the prime meridian and the
;			  equator should be labeled in degrees. If LABELS is
;			  given a value of 2, i.e. LABELS=2, then the longitude
;			  labels will be in hours and minutes instead of
;			  degrees.
;       CHRTHK          = Thickness of the lat. and long. labels.
;       CHRCOL          = Color of the lat. and long. labels.
;       CHRSIZ          = Size of the lat. and long. labels.
;
;                         Any keyword allowed by "oplot".
;
; OUTPUTS:
;	Draws grid lines on current graphics device.
;
; CATAGORY:
;	SST-9 Graphics routine.
;
; AUTHOR AND MODIFICATIONS:
;
;	J. Bloch 1.2  06/02/91
;	LN,      1.3  20/11/01 From AITOFF_GRID
;
;-

  if not keyword_set(n) then n=0
  if not KEYWORD_SET(LINECOLOR) THEN linecolor=!P.color
  if not KEYWORD_SET(CHRTHK) THEN chrthk=!P.charthick
  if not KEYWORD_SET(CHRTHK) THEN chrsiz=!P.charsize
  if not KEYWORD_SET(CHRCOL) THEN chrcol=!P.color

	if n_params() lt 2 then dlong = 30.0
	if n_params() lt 1 then dlat = 30.0
;
;	Do lines of constant longitude
;
	lat=findgen(181)-90
	lng=fltarr(181)
	lngtot = long(360.0/dlong)
	for i=0,lngtot do begin
		lng(*)=-180.0+(i*dlong)
		aitoff,lng,lat,x,y
		oplot,x,y,linestyle=n, color=linecolor, _extra=e
	endfor
;
;	Do lines of constant latitude
;
	lng=findgen(360)-180.0
	lat=fltarr(360)
	lattot=long(180.0/dlat)
	for i=1,lattot do begin
		lat(*)=-90+(i*dlat)
		aitoff,lng,lat,x,y
		oplot,x,y,linestyle=n, color=linecolor, _extra=e
	endfor
;
;	Do labeling if requested
;
	if keyword_set(label) then begin
;
;	Label equator
;
	    for i=0,lngtot-1 do begin
		lng = (i*dlong)
		if (lng ne 0.0) and (lng ne 180.0) then begin
		    aitoff,lng,0.0,x,y
		    if label eq 1 then xyouts,x(0),y(0), $
                        charthick=chrthk, color=chrcol, chars=chrsiz, $
			strcompress(string(lng,format="(I4)"),/remove_all) $
		    else begin
		        tmp=sixty(lng*24.0/360.0)
		        xyouts,x(0),y(0), charthick=chrthk, color=chrcol, $
			    chars=chrsiz, strcompress(string(tmp(0),tmp(1),$
			    format='(I2,"h",I2,"m")'),/remove_all),alignment=0.5
		    endelse
		endif
	    endfor
;
;	Label prime meridian
;
	    for i=1,lattot-1 do begin
		lat=-90+(i*dlat)
		aitoff,0.0,lat,x,y
		xyouts,x(0),y(0), charthick=chrthk, color=chrcol, chars=chrsiz,$
			strcompress(string(lat,format="(I4)"),/remove_all)
	    endfor
	endif

	return
end
