@mcs_usrlib   ; User contributed Functions and Pro

PRO gwgc_aitoff, PS=ps, ENC=enc, PORT=port, MONO=mono, GRID=grid, TOPLABEL=toplabel
;+
; NAME:
;       MESSIER_AITOFF
;
; PURPOSE:
;   MCS demonstration program using the routines in the "mcs_usrlib" library.
;
;   Query the Messier table from the DIF DB and store coords in a local array
;   via DBTabFLT.
;   Objects positions are plotted on the sky (aitoff projection).
;
; CATEGORY:
;   MCS demonstration program using the mcs_usrlib library.
;
; CALLING SEQUENCE:
;   messier_aitoff
;
; INPUT PARAMETERS:
;   None.
;
; INPUT KEYWORDS:
;      PS: if set then plot to "aitoff_sky.ps"
;     ENC: if set then use Encapsulated PS file
;    PORT: if set then use portrait orientated PS file
;    MONO: if set then plot in B/W
;    GRID: if set then overplot RA/DEC grid
;  TOPLAB: mark plot with the given top label
;
; NOTES:
;   Requires the IDL Astronomy user's library
;   (see http://idlastro.gsfc.nasa.gov/).
;   and the custom functions "aitoff_grid2" and "loadct19". 
;
; MODIFICATION HISTORY:
;   Written by L. Nicastro @ IASF-INAF, July 2006.
;  03/12/2014: LN, changed for the IMPRS DB course.
;-

; This is the common section for the "mcs_usrlib"
;COMMON DBinfo
;COMMON TABinfo

; Here we could use the common block parameters instead of passing them.
; Be sure the user can read the table. To gibe read permission, as root:
; mysql> grant select on DIF.* to mpe@localhost;

  c_user  = 'mpe'  &  c_password = 'mpe2014pass'  &  c_dbname = 'test'

; Now we select a subset of the columns and save them into a float matrix

  ;c_query = 'SELECT RAJ2000, DEJ2000, Bmag_A FROM pp2tdec'
  ;c_query = 'SELECT RAJ2000, DEJ2000, Bmag_A FROM gwgc'
  c_query = 'SELECT RAJ2000, DEJ2000, Bmag_A FROM pp500dec'
  status = DBExec_Qry(c_dbname,c_user,c_password,c_query)

; Save all the data into a float matrix
  d = DBTabFLT()
  ;help, d
  ;print, d[0,0:2]

  DBclose

  ra1 = d[0,*]
  dec1 = d[1,*]
  ns1 = N_ELEMENTS(ra1)
  gl1 = ra1
  gb1 = dec1

; Define a 19 color table
  c = loadct19(ps=ps,mono=mono)

  tf = 1
  xyouts, 0,0,'!17',/norm     ; dummy call to set vector font 17
  pcsav = !P.COLOR           ; save for restoring
  pbsav = !P.BACKGROUND      ; save for restoring
  if (keyword_set(mono) and not keyword_set(ps)) then $
    c[0:17] = 0;

  !P.COLOR = c[0]
  !p.background = c[18]

  if keyword_set(ps) then begin
    tf = 2
    odev = !d.name
    !p.color = 0
    set_plot, 'ps'
    device, file='gwgc_sky.ps', /color, ENCAP=enc
; Portrait or Landscape but preserving the aspect ratio 26./14 = 19.5/10.5
    if keyword_set(port) then device, /port,xoff=0,yoff=10,xsize=19.5,ysize=10.5 $
    else device, /land,yoff=28,xsize=26,ysize=14
    device, /times, font_index=17
    device, /helvetica, /bold, /oblique, font_index=18
  endif else begin
    ps = 0
    window, xs=900, ys=500
  endelse

; From RA and DEC to Gl, Gb
  for i=0, ns1-1 do begin
    glactc, ra1[i]/15.,dec1[i],2000.,l,b,1
    gl1[i] = l
    gb1[i] = b
  endfor

  aitoff, gl1,gb1,x1,y1

; Filled circle
  pp = findgen(32)*(!pi*2/31.)
  usersym, cos(pp),sin(pp),/fill
;
  ;plot, x1,y1,psym=8,xstyle=1,ystyle=1,color=c[2]  ;, back=5
  plot, x1,y1,psym=8,xstyle=4,ystyle=4,color=c[2]  ;, back=5

  aitoff_grid2, 30,15,label=1       ; AITOFF grid with corner values

  plots, 0.30,0.05, psym=8, symsize=1.5, color=c[2], /norm
  xyouts, 0.31,0.03,'!17GWGC Objects',charsize=1.5, charthick=tf, font=ps-1, /norm

; Draws the DEC = 30 strip
  gl1 = fltarr(200)  &  gb1 = gl1  &  gb2 = gl1  &  gl2 = gl1
  ra1 = findgen(200) * 24./199.
  for i = 0,199 do begin
    glactc, ra1(i),30,2000.,l,b,1
    gl1[i] = l
    gb1[i] = b
  endfor

  ii = sort(gl1)
  gl1 = gl1[ii]
  gb1 = gb1[ii]
  i = where(gl1 gt 180)
  i = i[0]
  gl1 =[gl1[i:199],gl1[0:i-1]]
  gb1 = [gb1[i:199],gb1[0:i-1]]

  aitoff, gl1,gb1,x1,y1
  plots, x1,y1,color=c[4],thick=tf*4

  !P.COLOR = pcsav          ; restore
  !P.BACKGROUND = pbsav     ; restore

  if keyword_set(ps) then begin
    device, /close
    set_plot, odev
    print, 'File "gwgc_sky.ps" ready.'
  endif

return
end
