Function loadct19, Colnames, PS=ps, MONO=mono, BOTTOM=bottom
;+
; Return a 19 elements integer array with the color table:
;  black, white, red, green, blue, cyan, magenta, yellow,
;  orange, green+yellow, green+cyan, blue+cyan, blue+magenta, red+magenta
;  DarkGray, LightGray, DarkRed, DarkGreen, DarkBlue
;
; If "ps" is set then col(0) = 0 (black) else 1 (white).
; If "mono" is set then all colors are set = col(0).
; If "colnames" is given then returns a string array with color names.
; If "bottom" is set then allocate table starting from this index
;
; LN@IASF-INAF
;-

COMMON COLORS, R_ORIG, G_ORIG, B_ORIG, R_CURR, G_CURR, B_CURR

if (N_ELEMENTS(r_orig) eq 0 or KEYWORD_SET(ps)) then $
  loadct, 0, /SILENT     ; Load B/W color table to set !D.TABLE_SIZE,
                         ; COMMON COLORS, etc.

if (n_elements(bottom) eq 0) then bottom = 0B
if (n_elements(ncolors) eq 0) then ncolors = !d.table_size - bottom

;print, bottom, ncolors

;- Get IDL version number
version = float(!version.release)

;-------------------------------------------------------------------------------
;- GET BIT DEPTH FOR THIS DISPLAY
;-------------------------------------------------------------------------------

;- If this device supports windows, make sure a window has been opened
if (!d.flags and 256) ne 0 then begin
  if (!d.window lt 0) then begin
    window, /free, /pixmap
    wdelete, !d.window
  endif
endif

;- Set default display depth
depth = 8

;- Get actual bit depth on supported displays
if (!d.name eq 'WIN') or (!d.name eq 'MAC') or (!d.name eq 'X') then begin
  if (version ge 5.1) then begin
    device, get_visual_depth=depth
  endif else begin
    if (!d.n_colors gt 256) then depth = 24
  endelse
endif

;-------------------------------------------------------------------------------
;- SELECT DECOMPOSED COLOR MODE (ON OR OFF) FOR 24-BIT DISPLAYS
;-------------------------------------------------------------------------------

if (!d.name eq 'WIN') or (!d.name eq 'MAC') or (!d.name eq 'X') then begin
  if (depth gt 8) then begin
;    if (version ge 5.2) then device, get_decomposed=entry_decomposed else $
;      entry_decomposed = 0
;    if (true eq 1) or (channel gt 0) then device, decomposed=1 else $
      device, decomposed=0
  endif
endif

nc = MIN([19,!D.TABLE_SIZE])
col = INTARR(nc)
; Define 19 colors:
;  black, white, red, green, blue, cyan, magenta, yellow, orange,
;  green+yellow, green+cyan, blue+cyan, blue+magenta, red+magenta
;  DarkGray, LightGray, DarkRed, DarkGreen, DarkBlue
  r = [0, 255, 255,   0,   0,   0, 255, 255, 255, 128,   0,   0, 128, 255, $
    85, 170, 128,   0,   0]
  g = [0, 255,   0, 255,   0, 255,   0, 255, 128, 255, 255, 128,   0,   0, $
    85, 170,   0, 128,   0]
  b = [0, 255,   0,   0, 255, 255, 255,   0,   0,   0, 128, 255, 255, 128, $
    85, 170,   0,   0, 128]

if (N_PARAMS() gt 0) then begin
  colnames = ['White','White','Red','Green','Blue','Cyan','Magenta','Yellow', $
              'Orange','Green+Yellow','Green+Cyan','Blue+Cyan', $
              'Blue+Magenta','Red+Magenta','DarkGray','LightGray','DarkRed', $
              'DarkGreen','DarkBlue']
  if KEYWORD_SET(ps) then colnames[0] = 'Black'
endif

  tvlct, r, g, b, bottom
  for i = 0, nc - 1 do begin
    R_ORIG[i] = r[i]  &  R_CURR[i] = r[i]
    G_ORIG[i] = g[i]  &  G_CURR[i] = g[i]
    B_ORIG[i] = b[i]  &  B_CURR[i] = b[i]
  endfor

  ;tvlct, [0, 246,  95,  21,  87, 244, 255, 255,   0,   0, 255], $
  ;       [0,  34, 251,  53, 254,  62, 252, 131, 131,   0, 255], $
  ;       [0,  23,  23, 255, 255, 255, 23,    0,   0, 131, 255]

if KEYWORD_SET(ps) then col[0] = 0 else col[0] = 1
if not KEYWORD_SET(mono) then begin
 for i = 1, nc -1 do col[i] = i
endif else begin
 col[1:nc-1] = col[0]
endelse

return, col
end
